
class Rover:
    
    def __init__(self, x, y, tile, battery=100):
        """
        Initialises the rover
        """
        self.visited = []
        self.full_battery = battery
        self.battery = battery
        
        self.set_tile(tile, (x, y))
        
        
    def set_tile(self, tile, location):
        self.tile = tile
        self.x, self.y = location
        self.tile.set_occupant(self)
        if location not in self.visited:
            self.visited.append(location)
    
    
    def move(self, direction, cycles, planet):
        """
        Moves the rover on the planet
        """
        direction = direction.lower()
        for i in range(cycles):
            next_x = self.x
            next_y = self.y
            if direction == 'n':
                next_x = self.x - 1
            elif direction == 's':
                next_x = self.x + 1
            elif direction == 'w':
                next_y = self.y - 1
            elif direction == 'e':
                next_y = self.y + 1
            next_x = next_x % planet.height
            next_y = next_y % planet.width
            next_tile = planet.tiles[next_x][next_y]
            if not self.has_battery() and next_tile.is_shaded():
                break
            if (self.tile.elevation() == next_tile.elevation() or self.tile.elevation() == next_tile.elevation_lo 
                    or next_tile.elevation() == self.tile.elevation_lo):
                self.set_tile(next_tile, (next_x, next_y))
                if self.tile.is_shaded():
                    self.battery = self.battery - 1
            else:
                break
    
    def wait(self, cycles):
        """
        The rover will wait for the specified cycles
        """
        for i in range(cycles):
            if self.tile.is_shaded() or self.battery == self.full_battery:
                break
            self.battery = self.battery + 1
    
    def visited_tiles(self):
        """
        Returns the number of tiles the rover explored so far.
        """
        return len(self.visited)
        
    def has_battery(self):
        """
        Returns True if rover's battery charge is non-zero
        """
        return self.battery > 0
